import argparse
from pathlib import Path
import pandas as pd

from .read import read_onelist_dataframe


def yaml2csv(inpath, outpath=None, encoding='utf-8'):
    inpath = Path(inpath)
    suffix = inpath.suffix
    if outpath:
        outpath = Path(outpath)
    else:
        outpath = inpath.with_suffix('.yaml')

    with open(inpath, 'r') as f:
        df = read_onelist_dataframe(f)
    df.to_csv(outpath, encoding=encoding, index=False)
    print("Done writing CSV file: {}".format(outpath.name))


def parse_arguments():
    parser = argparse.ArgumentParser(description='Read CSV file using Pandas read_csv() function.')

    parser.add_argument('file_path', type=str, help='Path to the CSV file')


    # Optional arguments
    parser.add_argument('-o', '--output', type=str, default=None,
                        help='Character indicating the start of a comment line in the CSV file')
    parser.add_argument('-e', '--encoding', type=str, default='utf-8',
                        help='Character indicating the start of a comment line in the CSV file')

    return parser.parse_args()

def main():
    # Check if the script is being run as the main program
    # Parse command line arguments
    args = parse_arguments()

    # Read CSV file using Pandas read_csv() function
    try:
        yaml2csv(args.file_path, args.output, args.encoding)
    except FileNotFoundError:
        print(f"Error: File not found at {args.file_path}")
    except pd.errors.EmptyDataError:
        print(f"Error: The CSV file at {args.file_path} is empty")


# Run the main function
if __name__ == "__main__":
    main()



import pandas as pd
from ast import literal_eval

if __name__ == "__main__":
    from common import Entry, NaN
else:
    from .common import Entry, NaN


class Read():
    FIRST_CHAR2TYPES = {
        '[': list,
        '"': str,
        "'": str,
    }

    LIST_CAST_TYPES = {
        (int, float): float,
        (float, int): float
    }


    def __init__(self, is_onelist=False, tgt_parent=None):
        self.is_onelist = is_onelist
        self.tgt_parent = tgt_parent
        
        #init read
        self.out = None
        self.current_parent = None
        self.list_counter = 0
        self.yaml_obj = dict()
        self.yaml_obj_types = dict()
        self.all_types = dict()
        self.list_counter = 0

        #line state
        self.key = None
        self.value = None
        self.is_parent = False
        self.is_child = False
        self.is_minus = False
        self.is_obj_parsing_done = False
        self.is_parent_value = False
        self.is_obj_parsing = False
        self.is_2ndimen_parsing = False
    
    @classmethod
    def _ylist_type_cast(cls, old_types, new_types):
        def c(fromto):
            from_type, to_type = fromto
            if to_type is type(None):
                return from_type

            if from_type == to_type:
                return from_type
            else:
                new_type =  cls.LIST_CAST_TYPES.get(fromto, False)
                if not new_type:
                    raise Exception(f'List type error, tyring to cast {from_type} to {to_type}')
                return new_type
        
        return map(c, zip(old_types.values(), new_types.values()))

    @classmethod
    def infer_type(cls, value):
        ytype = cls.FIRST_CHAR2TYPES.get(value[0], False)
        if not ytype:
            if value == NaN:
                ytype = type(None)
            else:
                ytype = float if '.' in value else int
        return ytype

    def _reset(self):
        #reset and skip
        if self.is_parent:
            if self.is_onelist and self.list_counter:
                raise Exception("You specified a one list('-') yaml2d file but a key was found after parsing the list")
            if self.is_parent_value:
                self.current_parent = None
            else:
                self.current_parent = self.key
            self.list_counter = 0
            
        if self.is_obj_parsing_done:
            self.yaml_obj = dict()
            self.yaml_obj_types = dict()
            


    def process_line(self, line):
        striped_line = line.strip()
        if not striped_line:
            return None

        self.is_child = line[0].isspace()
        self.is_parent = not self.is_child
        line = striped_line

        key, value = line.strip().split(':', 1)
        self.key, self.value = key.strip(), value.strip()
        
        self.is_parent_value = self.is_parent and self.value

        self.is_minus = key[0] == '-'
        self.is_obj_parsing_done = (self.is_parent or self.is_minus) and bool(self.yaml_obj)
        
        #states
        if self.is_child:
            self.is_obj_parsing = True
        if self.is_parent:
            self.is_obj_parsing = False

        if self.is_obj_parsing:
            self.is_2ndimen_parsing = True
        if self.is_parent:
            self.is_2ndimen_parsing = False
        
        
    def parsing_obj(self):
        #record current line if not parent
        if self.is_minus:
            self.list_counter += 1
            self.key = self.key[1:].strip()
        self.yaml_obj[self.key] = self.value
        ytype = self.infer_type(self.value)        
        self.yaml_obj_types[self.key] = ytype


    def read_obj(self):
        #write previous object to the self.backend if done parsing
        if self.list_counter and self.current_parent in self.all_types:
            new_obj_types = self.all_types[self.current_parent]
            new_obj_types = self._ylist_type_cast(self.all_types[self.current_parent], self.yaml_obj_types)
            self.yaml_obj_types = dict(zip(self.yaml_obj_types.keys(), new_obj_types))
        self.all_types[self.current_parent] = self.yaml_obj_types
        return True


    def read_generator(self, lines):
        for line in lines:
            self.process_line(line)
            if self.is_obj_parsing_done:
                self.read_obj()
                yield Entry(
                    parent=self.current_parent,
                    obj=self.yaml_obj,
                    ytype=self.yaml_obj_types,
                    is_ylist= bool(self.list_counter),
                    is_parent_value= False,
                    is_last=False
                )
            if self.is_parent_value:
                yield Entry(
                    parent= self.key,
                    obj=self.value,
                    ytype= self.infer_type(self.value),
                    is_parent_value= True,
                    is_ylist=False,
                    is_last=False
                )
            self._reset()
            if self.is_obj_parsing:
                self.parsing_obj()
        if self.is_obj_parsing:
            self.read_obj()
            yield Entry(
                obj=self.yaml_obj,
                ytype=self.yaml_obj_types,
                is_ylist= bool(self.list_counter),
                is_parent_value= False,
                is_last=False
            )
            yield Entry(is_last=True)
        

def _python_eval(value):
    if value == NaN:
        return None
    return literal_eval(value)


def read_onelist_meta(lines):
    read = Read(is_onelist=False, tgt_parent=None)
    out = {}
    for entry in read.read_generator(lines):
        if entry.is_ylist:
            return out
        if entry.is_parent_value:
            out[entry.parent] = _python_eval(entry.obj)
        else:
            out[entry.parent] = {k: _python_eval(v) for k, v in entry.obj.items()}
                 

def read_onelist_generator(lines, transform=None):
    read = Read(is_onelist=False, tgt_parent=None)
    def gen():
        for entry in read.read_generator(lines):
            if not entry.is_ylist:
                continue
            tmp = {k: _python_eval(v) for k, v in entry.obj.items()}
            if transform:
                tmp = transform(tmp)
            yield tmp
    return gen()


def read_onelist_dataframe(lines):
    read = Read(is_onelist=False, tgt_parent=None)
    data = {}
    gen = read_onelist_generator(lines)
    for entrydict in gen:
        if not data:
            data = {k:[v] for k,v in entrydict.items()}
        else:
            try:
                for k, v in entrydict.items():
                    data[k].append(v)
            except KeyError as e:
                raise Exception('Probably violated YAML (-)list must contain fixed features: ' + e.message) from e
    df = pd.DataFrame(data)
    return df


if __name__ == "__main__":
    yamlf = """
config1:
  key1: 'value1'
  key2: 'value2'
  key3: 'value3'

config2:
  keyA: 'valueA'
  keyB: 'valueB'
  keyC: 'valueC'

data:
  - name: 'John Doe'
    age: 30
    city: 'New York'
  - name: 'Jane Smith'
    age: 25
    city: 'San Francisco'
  - name: 'Bob Johnson'
    age: 35
    city: 'Chicago'
  - name: 'Test'
    age: 35.0
    city: 'Chicago'
    """

    out = read_onelist_meta(yamlf.splitlines())
    print(out)
    gen = read_onelist_generator(yamlf.splitlines())

    for item in gen:
        print(item)



from numbers  import Number


TAB = '  '
MINUS_TAB = '- '
NL = '\n'
SEP = ': '
NaN = 'null'

def infer_repr(value):
    str_value = repr(value).lower()
    if str_value == 'nan' or str_value == 'none':
        return NaN
    if isinstance(value, Number):
        return str(value)
    if isinstance(value, str):
        #return '"{}"'.format(repr(value))
        return repr(value)
    if isinstance(value, list):
        #TODO !!
        return str(value)


class Entry():
    def __init__(self, parent=None, obj=None, ytype=None, is_ylist=None, is_parent_value=None, is_last=False):
        self.is_last = is_last
        self.parent = str(parent)
        self.obj = obj
        self.ytype = ytype
        self.is_ylist = is_ylist
        self.is_parent_value = is_parent_value
        self.is_write_ready = False
        
    @classmethod
    def from_dict(cls, obj, parent, is_ylist=False):
        this = cls()
        this.parent = parent
        this.is_ylist = is_ylist
        this.obj = obj
        this.obj = {str(k): infer_repr(v) for k,v in this.obj.items()}
        this.is_write_ready = True
        return this
        
    @classmethod
    def from_keyval(cls, key, val):
        key = str(key)
        this = cls()
        this.parent= key
        this.is_ylist = False
        this.is_parent_value = True
        this.obj = infer_repr(val)
        this.is_write_ready = True
        return this
        
    @classmethod
    def dict2d_to_list(cls, dict2d):
        return [cls.from_keyval(k,v) if not isinstance(v, dict) else 
                        cls.from_dict(v, parent=k, is_ylist=False)  
                        for k, v in dict2d.items()]
            



import itertools 

if __name__ == "__main__":
    from common import Entry, TAB, MINUS_TAB, NL, SEP, NaN
else:
    from .common import Entry, TAB, MINUS_TAB, NL, SEP, NaN


class Write():
    def __init__(self):
        self.buffer = ""
        self.last_keys = None
        self.max_buffer_size = 400
        self.is_writing_list = False
    
    def write_entry(self, entry):
        if entry.is_parent_value:
            self.buffer += entry.parent + SEP + entry.obj + NL
            self.buffer += NL
        elif entry.is_ylist:
            if not self.is_writing_list and entry.parent:  
                #parent check incase of append
                self.buffer += entry.parent + SEP + NL
            for i, keyval in enumerate(entry.obj.items()):
                key, val = keyval
                if i == 0:
                    key = MINUS_TAB + key
                else:
                    key = TAB + key
                item = TAB + key + SEP + val
                self.buffer += item + NL
        else:
            self.buffer += entry.parent + SEP + NL
            self.last_keys = entry.obj.keys()
            for i, keyval in enumerate(entry.items()):
                key, val = keyval
                item = TAB + key + SEP  + val
                self.buffer += item + NL
            self.buffer += NL
            
        self.is_writing_list = entry.is_ylist


    def write(self, path, entries, flag='w'):
        with open(path, flag) as f:
            for entry in entries:
                self.write_entry(entry)
                if len(self.buffer) > self.max_buffer_size:
                    f.write(self.buffer)
                    self.buffer = ""
            f.write(self.buffer)

            
def write_dataframe(path, df, name):
    itr = df.iterrows()
    itr = df.to_dict(orient="records")
    itr = map(lambda x: Entry.from_dict(parent=name, obj=x, is_ylist=True), itr)
    
    if df.attrs:
        itr = itertools.chain(Entry.dict2d_to_list(df.attrs), itr)

    write = Write()
    write.write(path, itr)


def write_dict2d(path, dict2d):
    itr = Entry.dict2d_to_list(df.attrs)
    write = Write()
    write.write(path, itr)

def append_write(path, dict1d):
    entry = Entry.from_dict(dict1d, parent=None, is_ylist=True)
    write = Write()
    write.write(path, [entry], flag='a')

if __name__ == "__main__":
    import pandas as pd

    data = {'Name': ['Alice', 'Bob', 'Charlie'],
            'Age2': [25, 30, 22],
            'Age':  [25, 30, 22],
            'City': ['New York', 'San Francisco', 'Los Angeles']}

    df = pd.DataFrame(data)
    df.attrs = {'x': 32323.3}
    df.attrs['liest'] = [12,33,45,353]
    write_dataframe('./out.yaml', df, "dataie")
    write_dict2d('./dict2d.yaml', df.attrs)
    append_write('./out.yaml', {"Appended": "Appended",
    "Age2": 22,
    "Age": 22,
    "City": "Los Angeles"})



from ast import literal_eval

if __name__ == "__main__":
    from read import Read
else:
    from .read import Read


def _python_eval(value):
    return literal_eval(value)

def _python_generator_backend(out, is_last, current_parent, yaml_obj,  list_counter, ytypes):
    if is_last:
        return out
    yaml_obj = {k: _python_eval(v) for k, v in yaml_obj.items()}
    return yaml_obj


def read2gen(stryaml2d):
    lines = stryaml2d.splitlines()
    read = Read( _python_generator_backend, is_onelist=False, tgt_parent='data')
    return read.read_generator(lines)


if __name__ == "__main__": 
    yamlf = """
config1:
  key1: 'value1'
  key2: 'value2'
  key3: 'value3'

config2:
  keyA: 'valueA'
  keyB: 'valueB'
  keyC: 'valueC'

data:
  - name: 'John Doe'
    age: 30
    city: 'New York'
  - name: 'Jane Smith'
    age: 25
    city: 'San Francisco'
  - name: 'Bob Johnson'
    age: 35
    city: 'Chicago'
  - name: 'Test'
    age: 35.0
    city: 'Chicago'
    """
    out = read2gen(yamlf)
    for item in out:
        print(item)


# setup.py

from setuptools import setup, find_packages

setup(
    name='yaml2d',
    version='0.0.10',
    author='Yahya Badran',
    author_email='techtweaking@gmail.com',
    description='parser of YAML2D, a tiny subset of YAML',
    url='https://github.com/badranx/YAML-2D',
    install_requires=['pandas >= 1.0.0'],
    packages=find_packages(),
    license='MIT',
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import pandas as pd
import argparse
from pathlib import Path

from .write import write_dataframe


def df2yaml(df, inpath, outpath=None):
    inpath = Path(inpath)
    suffix = inpath.suffix
    if outpath:
        outpath = Path(outpath)
    else:
        outpath = inpath.with_suffix('.yaml')

    write_dataframe(outpath, df, name='data')
    print("Done writing YAML file: {}".format(outpath.name))


def parse_arguments():
    parser = argparse.ArgumentParser(description='Read CSV file using Pandas read_csv() function.')

    parser.add_argument('file_path', type=str, help='Path to the CSV file')


    # Optional arguments
    parser.add_argument('-o', '--output', type=str, default=None,
                        help='Character indicating the start of a comment line in the CSV file')
    parser.add_argument('-s', '--separator', type=str, default=',',
                        help='Separator used in the CSV file (default is comma)')

    parser.add_argument('-c', '--columns', default=None, nargs='+',
                        help='List of columns to select from the CSV file')

    parser.add_argument('-H', '--header', default='infer', action='store_true',
                        help='Specify if the CSV file has a header row')

    parser.add_argument('--comment', type=str, default=None,
                        help='Character indicating the start of a comment line in the CSV file')

    return parser.parse_args()

def main():
    # Check if the script is being run as the main program
    # Parse command line arguments
    args = parse_arguments()

    # Read CSV file using Pandas read_csv() function
    try:
        df = pd.read_csv(args.file_path, sep=args.separator, header=0 if args.header else None,
                         usecols=args.columns, comment=args.comment)
        df2yaml(df, args.file_path, args.output)
    except FileNotFoundError:
        print(f"Error: File not found at {args.file_path}")
    except pd.errors.EmptyDataError:
        print(f"Error: The CSV file at {args.file_path} is empty")


# Run the main function
if __name__ == "__main__":
    main()
